package com.appconsentclearreactnative

import android.util.Log
import com.facebook.react.bridge.Promise
import com.facebook.react.bridge.ReactApplicationContext
import com.facebook.react.bridge.ReactContextBaseJavaModule
import com.facebook.react.bridge.ReactMethod
import com.facebook.react.bridge.ReadableMap
import com.sfbx.appconsentv3.AppConsent
import com.sfbx.appconsentv3.ui.AppConsentSDK
import com.sfbx.appconsentv3.ui.listener.OnPresentNoticeListener
import com.sfbx.appconsentv3.ui.model.ACConfiguration
import com.sfbx.appconsentv3.ui.model.ACConsentStatus
import com.sfbx.appconsentv3.ui.model.ACConsentableType

class AppconsentClearReactnativeModule(private val reactContext: ReactApplicationContext) :
  ReactContextBaseJavaModule(reactContext) {

  override fun getName(): String {
    return "AppconsentClearReactnative"
  }

  private var appconsentUi: AppConsent? = null
  private var appKey: String? = null
  private var forceApplyGDPR: Boolean = false

  override fun getConstants(): MutableMap<String, Any> {
    return mutableMapOf("version" to "5.0.0")
  }

  @ReactMethod
  fun requiresMainQueueSetup(): Boolean {
    return true
  }

  @ReactMethod
  fun configureWith(appKey: String, forceApplyGDPR: Boolean, forceATT: Boolean, promise: Promise) {
    this.appKey = appKey
    this.forceApplyGDPR = forceApplyGDPR
    try {
      AppConsentSDK.initialize(
        appKey = appKey,
        configuration = ACConfiguration.Builder()
          .setForceApplyGDPR(forceApplyGDPR)
          .build()
      ) {
        appconsentUi = it
        promise.resolve(true)
      }
    } catch (ex: Throwable) {
      Log.e(
        AppconsentClearReactnativeModule::class.simpleName,
        "An error occurred during the configuration of the module: ${ex.message}"
      )
      promise.reject(ex)
    }
  }

  @ReactMethod
  fun present(force: Boolean) {
    appconsentUi?.tryToDisplayNotice(force)
  }

  @ReactMethod
  fun checkForUpdate(promise: Promise) {
    appconsentUi?.checkForUpdate(onResult = {
      promise.resolve(it)
    },
      error = {
        promise.reject(it)
      })
  }

  @ReactMethod
  fun isGDPRCountry(promise: Promise) {
    promise.resolve(appconsentUi?.isSubjectToGDPR())
  }

  @ReactMethod
  fun consentAlreadyGiven(promise: Promise) {
    promise.resolve(appconsentUi?.consentGiven())
  }

  @ReactMethod
  fun consentGiven(promise: Promise) {
    appconsentUi?.setOnPresentNoticeListener(object : OnPresentNoticeListener {
      override fun presentConsentError(error: Throwable?) {
        promise.reject(error ?: Throwable("An unknown error has occurred."))
      }

      override fun presentConsentGiven() {
        promise.resolve(appconsentUi?.consentGiven())
      }
    })
  }

  @ReactMethod
  fun clearConsent() {
    appconsentUi?.clearConsent()
  }

  @ReactMethod
  fun extraConsentableAllowed(extraId: String?, promise: Promise) {
    promise.resolve(appconsentUi?.extraConsentableAllowed(extraId!!))
  }

  @ReactMethod
  fun extraVendorAllowed(extraId: String?, promise: Promise) {
    promise.resolve(appconsentUi?.extraVendorAllowed(extraId!!))
  }

  @ReactMethod
  fun extraFloatingPurposeAllowed(extraId: String?, promise: Promise) {
    promise.resolve(appconsentUi?.extraFloatingAllowed(extraId!!))
  }

  @ReactMethod
  fun setConsentableConsent(values: ReadableMap, promise: Promise) {
    val map = values.toHashMap().entries.associate { entry ->
      entry.key.toInt() to ACConsentStatus.values()
        .first { it.ordinal == (entry.value as Double).toInt() }
    }
    appconsentUi?.setConsentableConsents(map,
      success = {
        promise.resolve(true)
      },
      error = {
        promise.reject(it)
      })
  }

  @ReactMethod
  fun setExtraConsentableConsent(values: ReadableMap, promise: Promise) {
    val map = values.toHashMap().mapValues { entry ->
      ACConsentStatus.values().first { it.ordinal == (entry.value as Double).toInt() }
    }
    appconsentUi?.setExtraConsentableConsents(map,
      success = {
        promise.resolve(true)
      },
      error = {
        promise.reject(it)
      }
    )
  }

  // Deprecated
  @ReactMethod
  fun consentableAllowedByObjectId(id: String, promise: Promise) {
    val intId = id.toInt()
    promise.resolve(appconsentUi?.consentableAllowed(intId, ACConsentableType.UNKNOWN))
  }

  @ReactMethod
  fun consentableAllowedByIABId(iabId: String, type: Int, promise: Promise) {
    val intId = iabId.toInt()
    val consentableType: ACConsentableType = when (type) {
      0 -> ACConsentableType.PURPOSE
      1 -> ACConsentableType.FEATURE
      2 -> ACConsentableType.SPECIAL_FEATURE
      3 -> ACConsentableType.SPECIAL_PURPOSE
      else -> ACConsentableType.UNKNOWN
    }
    promise.resolve(appconsentUi?.consentableAllowed(intId, consentableType))
  }

  @ReactMethod
  fun vendorAllowedByIABId(iabId: String, promise: Promise) {
    val intId = iabId.toInt()
    promise.resolve(appconsentUi?.vendorAllowed(intId))
  }

  @ReactMethod
  fun stackAllowedByIABId(iabId: String, promise: Promise) {
    val intId = iabId.toInt()
    promise.resolve(appconsentUi?.stackAllowed(intId))
  }

  @ReactMethod
  fun getAppKey(promise: Promise) {
    promise.resolve(appKey)
  }

  @ReactMethod
  fun getForceApplyGDPR(promise: Promise) {
    promise.resolve(forceApplyGDPR)
  }
}
