/**
 * This exposes the native AppConsent module as a JS module.
 *
 * @packageDocumentation
 */

import { NativeModules, Platform } from 'react-native';

const LINKING_ERROR =
  `The package 'appconsent-clear-reactnative' doesn't seem to be linked. Make sure: \n\n` +
  Platform.select({ ios: "- You have run 'pod install'\n", default: '' }) +
  '- You rebuilt the app after installing the package\n' +
  '- You are not using Expo managed workflow\n';

const appConsent = NativeModules.AppconsentClearReactnative
  ? NativeModules.AppconsentClearReactnative
  : new Proxy(
      {},
      {
        get() {
          throw new Error(LINKING_ERROR);
        },
      }
    );

/**
 * This is the main entry point to interact with AppConsent API.
 *
 * @example
 * import AppConsent from 'appconsent-clear-reactnative';
 */
export default appConsent as AppConsent;

/**
 * This is the main interface to interact with AppConsent API, it exposes typescript declarations to access Native libraries features.
 *
 * @example
 * import AppConsent from 'appconsent-clear-reactnative';
 */
export interface AppConsent {
  /**
   * This initializes AppConsent.
   * @param   appKey - Your AppKey
   * @param   forceApplyGDPR - force behavior as in a GDPR subject country
   * @param   forceATT - force ATT handling (iOS only)
   *
   * @category General Use
   */
  configureWith(
    appKey: string,
    forceApplyGDPR: boolean,
    forceATT: boolean
  ): void;

  /**
   * To display AppConsent UI by modal on top of your view.
   *
   * @param force - false displays introduction page first
   *
   * @category General Use
   */
  present(force: boolean): void;

  /**
   * Check if consent must be updated (new gvl version, new consentables…)
   *
   * @category Advanced Use
   */
  checkForUpdate(): Promise<boolean>;

  /**
   * Check if device is GDPR country.
   *
   * @returns true if the device is in a country subject to GDPR, false otherwise.
   *
   * @category Advanced Use
   */
  isGDPRCountry(): Promise<boolean>;

  /**
   * Check if user gave consent​
   *
   * @example
   * AppConsent.consentAlreadyGiven()
   *   .then( success =>
   *     console.log("🔥 consentAlreadyGiven => " + success)
   *    )
   *
   * @returns true if consent is given, false otherwise.
   *
   * @category Advanced Use
   */
  consentAlreadyGiven(): Promise<boolean>;

  /**
   * Callback when user give its consent.
   *
   * @returns true if user saved their consent preferences.
   *
   * @category Advanced Use
   */
  consentGiven(): Promise<boolean>;

  /**
   * Clear consents on NSUserDefaults, but not on server.
   *
   * @category Advanced Use
   */
  clearConsent(): void;

  /**
   *
   * @param extraId
   * @returns true if consentable with extraId is allowed, false otherwise.
   *
   * @category Advanced Use
   */
  extraConsentableAllowed(extraId: string): Promise<boolean>;

  /**
   *
   * @param extraId
   * @returns true if consentable with extraId is allowed, false otherwise.
   *
   * @category Advanced Use
   */
  extraVendorAllowed(extraId: string): Promise<boolean>;

  /**
   * Check if floating purpose is allowed.
   *
   * @remarks
   * NOTE:
   * You need to call checkForUpdate method before calling this method to stay up to date.
   *
   * @see {@link AppConsent.checkForUpdate}
   *
   * @param extraId
   *
   * @category Advanced Use
   */
  extraFloatingPurposeAllowed(extraId: string): Promise<boolean>;

  /**
   * Set consentables status with an iabId.
   *
   * @remarks
   * CAUTION:
   * Key must be an iabId, not an objectId.
   * Value must be an Integer: 0 (pending), 1 (allowed), -1 (denied)
   *
   * NOTE:
   * This method doesn’t send request to the server.
   *
   * @param values
   *
   * @category Advanced Use
   */
  setConsentableConsent(values: {}): Promise<boolean>;

  /**
   * Set consentables status with extraId.
   *
   * @remarks
   * CAUTION:
   * Key must be an extraId, not an objectId.
   * Value must be an Integer: 0 (pending), 1 (allowed), -1 (denied)
   *
   * NOTE:
   * This method doesn’t send request to the server.
   *
   * @param values
   *
   * @category Advanced Use
   */
  setExtraConsentableConsent(values: {}): Promise<boolean>;

  // IOS ONLY

  /**
   * @remarks
   * This is only supported on iOS targets.
   *
   * @returns forceATT
   *
   * @category iOS App Tracking Transparency
   */
  getForceATT(): Promise<boolean>;

  /**
   * Check if your device supports App Tracking Transparency.
   *
   * @remarks
   * This is only supported on iOS targets.
   *
   * @returns true if available, false otherwise.
   *
   * @category iOS App Tracking Transparency
   */
  appTrackingIsAvailable(): Promise<boolean>;

  /**
   * Check if App Tracking Transparency has been given.
   *
   * @remarks
   * This is only supported on iOS targets.
   *
   * @returns 0 (not given), 1 (given), 2 (not supported)
   *
   * @category iOS App Tracking Transparency
   */
  appTrackingAuthorizationGiven(): Promise<number>;

  /**
   * Check status of App Tracking Transparency.
   *
   * @remarks
   * This is only supported on iOS targets.
   *
   * @returns 0 (failure), 1 (success), 2 (not supported)
   *
   * @category iOS App Tracking Transparency
   */
  appTrackingAuthorizationStatus(): Promise<number>;

  // DEPRECATED

  /**
   * @returns true if consentable with objectId is allowed, false otherwise.
   *
   * @deprecated
   * @category Advanced Use
   */
  consentableAllowedByObjectId(id: string): Promise<boolean>;

  /**
   * @param iabId
   * @param type - 0: purpose, 1: feature, 2: specialFeature, 3: specialPurpose
   * @returns true if consentable with iabId is allowed, false otherwise.
   *
   * @deprecated
   * @category Advanced Use
   */
  consentableAllowedByIABId(iabId: string, type: number): Promise<boolean>;

  /**
   * @returns true if consentable with iabId is allowed, false otherwise.
   *
   * @deprecated
   * @category Advanced Use
   */
  vendorAllowedByIABId(iabId: string): Promise<boolean>;

  /**
   * @returns true if consentable with iabId is allowed, false otherwise.
   *
   * @deprecated
   * @category Advanced Use
   */
  stackAllowedByIABId(iabId: string): Promise<boolean>;

  /**
   * @returns the configured AppKey.
   *
   * @deprecated
   * @category Advanced Use
   */
  getAppKey(): Promise<string>;

  /**
   * @returns forceApplyGDPR
   *
   * @deprecated
   * @category Advanced Use
   */
  getForceApplyGDPR(): Promise<boolean>;

  /**
   *
   * @deprecated
   * @category Advanced Use
   */
  getName(): string;

  /**
   *
   * @deprecated
   * @category Advanced Use
   */
  getConstants(): any;
}
